"use strict";
const command_1 = require("@oclif/command");
const aws_sdk_1 = require("aws-sdk");
const chalk_1 = require("chalk");
class QueryFactory {
    static init(log) {
        const query = {};
        return {
            addRuntime(runtime) {
                log(`${chalk_1.default.green('Search condition')}: Runtime === ${runtime}`);
                query.Runtime = runtime;
                return this;
            },
            addSearchQuery(name) {
                log(`${chalk_1.default.green('Search condition')}: FunctionName contains ${name}`);
                query.name = name;
                return this;
            },
            getQuery() {
                return query;
            }
        };
    }
}
const regions = [
    "eu-north-1",
    "ap-south-1",
    "eu-west-3",
    "eu-west-2",
    "eu-west-1",
    "ap-northeast-2",
    "ap-northeast-1",
    "sa-east-1",
    "ca-central-1",
    "ap-southeast-1",
    "ap-southeast-2",
    "eu-central-1",
    "us-east-1",
    "us-east-2",
    "us-west-1",
    "us-west-2"
];
class LambdaFunctionSearch extends command_1.Command {
    constructor() {
        super(...arguments);
        this.NextMarker = '';
        this.amount = 0;
    }
    async listAllFunctions(query = {}, functions = []) {
        if (!this.client) {
            throw new Error('Failed to initilize AWS SDK Class');
        }
        const params = {};
        if (this.NextMarker)
            params.Marker = this.NextMarker;
        const { NextMarker, Functions } = await this.client.listFunctions(params).promise();
        this.NextMarker = NextMarker || '';
        const targetFunctions = !Functions ? [] : Functions.filter(func => {
            if (query.name) {
                const reg = new RegExp(query.name);
                if (func.FunctionName && !func.FunctionName.match(reg))
                    return false;
            }
            if (query.Runtime)
                return func.Runtime === query.Runtime;
            return true;
        });
        const items = functions.concat(targetFunctions);
        this.amount += Functions ? Functions.length : 0;
        if (this.NextMarker) {
            return this.listAllFunctions(query, items);
        }
        return items;
    }
    async getAllRegionFuncs(query, props = {}) {
        return await Promise.all(regions.map(region => {
            return this.worker(query, Object.assign({}, props, { region }));
        }));
    }
    async run() {
        const { flags } = this.parse(LambdaFunctionSearch);
        const queryBuilder = QueryFactory.init(this.log);
        if (flags.runtime)
            queryBuilder.addRuntime(flags.runtime);
        if (flags.search)
            queryBuilder.addSearchQuery(flags.search);
        const props = {};
        if (!flags.region)
            this.log(chalk_1.default.yellow('warning') + ': Missing region');
        if (flags.region) {
            if (flags.region === 'all')
                return this.getAllRegionFuncs(queryBuilder.getQuery(), props);
            props.region = flags.region;
        }
        return this.worker(queryBuilder.getQuery(), props);
    }
    async worker(query, props = {}) {
        const { flags } = this.parse(LambdaFunctionSearch);
        this.client = new aws_sdk_1.Lambda(props);
        if (flags.profile) {
            const credentials = new aws_sdk_1.SharedIniFileCredentials({ profile: flags.profile });
            this.client.config.credentials = credentials;
        }
        try {
            this.amount = 0;
            const result = await this.listAllFunctions(query);
            this.log(`=== ${chalk_1.default.green('Matched Functions')}: ${result.length} / ${this.amount} ===`);
            if (props.region)
                this.log(`${chalk_1.default.green('Region')} : ${props.region}`);
            result.forEach(item => {
                this.log(item.FunctionName);
                if (flags.showAll)
                    console.log(item);
            });
        }
        catch (e) {
            this.error(chalk_1.default.red(e));
            this.exit(1);
        }
    }
}
LambdaFunctionSearch.description = 'Search Lambda functions';
LambdaFunctionSearch.flags = {
    help: command_1.flags.help({ char: 'h' }),
    // runtime
    runtime: command_1.flags.string({
        char: 'R',
        description: [
            'Lambda runtime',
            'Example: ' + [
                "nodejs",
                "nodejs4.3",
                "nodejs6.10",
                "nodejs8.10",
                "java8",
                "python2.7",
                "python3.6",
                "python3.7",
                "dotnetcore1.0",
                "dotnetcore2.0",
                "dotnetcore2.1",
                "nodejs4.3-edge",
                "go1.x",
                "ruby2.5",
                "provided"
            ].join(', ')
        ].join('\n')
    }),
    // [For AWS SDK] region
    region: command_1.flags.string({
        char: 'r',
        description: 'region, (If you set "all", list all regions)'
    }),
    // [For AWS SDK] profile
    profile: command_1.flags.string({
        char: 'p',
        description: 'AWS CLI profile'
    }),
    search: command_1.flags.string({
        char: 's',
        description: 'search by name'
    }),
    showAll: command_1.flags.boolean({
        char: 'A',
        description: 'Show all function data',
        default: false
    }),
};
module.exports = LambdaFunctionSearch;
